"use client";

import {
  ActionIcon,
  Button,
  Container,
  Flex,
  Group,
  Modal,
  Space,
  Table,
  TextInput,
  Title,
  Tooltip,
} from "@mantine/core";
import { useDisclosure } from "@mantine/hooks";
import { IconDisabled, IconEye, IconMessage, IconSearch, IconTrash ,IconArrowsUpDown} from "@tabler/icons-react";
import axios, { AxiosError } from "axios";
import { useEffect, useState } from "react";
import { notifications } from "@mantine/notifications";

interface Message {
  note: string;
  time:string;
}

interface Ticket {
  ticket_id: string;
  category_of_request: string;
  nature_of_request: string;
  created_date: string;
  status: string;
  message: Message[];
}
interface TicketDetails {
  ticket_id: string;
  category_of_request: string;
  nature_of_request: string;
  additional_info: string;
  message : string;
  created_date: string;
  
}
export default function Page() {
  const [opened, { open, close }] = useDisclosure(false);
  const [tickets, setTickets] = useState<Ticket[]>([]);
  const [searchQuery, setSearchQuery] = useState("");
  const [filteredTickets, setFilteredTickets] = useState<Ticket[]>([]);
  const [activeMessages, setActiveMessages] = useState<Message[] | null>(null);
  const [ticketDetails, setTicketDetails] = useState<TicketDetails| null>(null);
  const [ticketStatus, setTicketStatus] = useState<string| null>(null);
  const [sortOrder,setSortOrder]= useState<"asc" |"desc">("desc");

  const [detailsOpened, { open: openDetails, close: closeDetails }] = useDisclosure(false);

  // For view ticket details
  const handleOpenDetails = async (ticketId: string) => {
    try {
      const response = await axios.get<TicketDetails>(`/api/ticket/${ticketId}`); // Assuming API provides details 
      setTicketDetails(response.data);
      openDetails();
    }
    catch(error: AxiosError | any) {
      console.error("Failed to fetch ticket details:", error);
      notifications.show({
          color: 'red',
          title: error.response.status,
          message: error.response.data.message
      })
    }
  };
  // For parsing additional_info
  const parseAdditionalInfo = (info: string) => {
    return info.split(", ").map((item) => {
      const [key, value] = item.split(": ");
      return { key: key.trim(), value: value?.trim() || "" };
    });
  };
  // For delete a ticket
  const handleDelete = async (ticketId: string) => {
    const isConfirm =window.confirm("Do you want to delete the ticket?");
    if(!isConfirm)
      return;
    try {
      const response = await axios.delete<TicketDetails>(`/api/ticket/${ticketId}`); // Assuming API provides details 
      const data = await response.data;
      alert(data.message);
    }
    catch(error: AxiosError | any) {
      console.error("Failed to delete ticket:", error);
      notifications.show({
          color: 'red',
          title: error.response.status,
          message: error.response.data.message
      })
    }
    finally{
      window.location.reload();
    }
  };
  // Sort and filter tickets based on the time 
  useEffect(() => {
    const sortedTickets = [...tickets].sort(
      // (a, b) => new Date(b.created_date).getTime() - new Date(a.created_date).getTime()
      (a, b) => parseInt(b.ticket_id) - parseInt(a.ticket_id)
    );
    const results = sortedTickets.filter((ticket) =>
      ticket.category_of_request.toLowerCase().includes(searchQuery.toLowerCase())
    );
    setFilteredTickets(results);
  }, [searchQuery, tickets]);

  // Fetch tickets from API
  useEffect(() => {
    async function fetchTickets() {
      try {
        const response = await axios.get<Ticket[]>("/api/ticket");
        setTickets(response.data);
      } catch (error) {
        console.error("Failed to fetch tickets: ", error);
      }
    }
    fetchTickets();
  }, []);

  // Sorting the ticket when any ticket resolved the go to the end of the list
  useEffect(() => {
    const sortedTickets = [...tickets].sort((a, b) => {
      if (a.status.toLowerCase() === "resolved" && b.status.toLowerCase() !== "resolved") return 1;
      if (b.status.toLowerCase() === "resolved" && a.status.toLowerCase() !== "resolved") return -1;
      return parseInt(b.ticket_id) - parseInt(a.ticket_id); // Sort by ticket ID otherwise
    });
    const results = sortedTickets.filter(ticket =>
      ticket.category_of_request.toLowerCase().includes(searchQuery.toLowerCase())
    ); 
    setFilteredTickets(results);
  }, [searchQuery, tickets]);

  // For sorting ticket history according to time
  const sortedMessages = activeMessages
  ? [...activeMessages].sort((a, b) => {
      const timeA = new Date(a.time).getTime();
      const timeB = new Date(b.time).getTime();
      return sortOrder === "asc" ? timeA - timeB : timeB - timeA;
    })
  : [];

  const handleOpenMessage = (messages: Message[],status:string) => {
    console.log(messages);
    setActiveMessages(messages);
    setTicketStatus(status);
    open();
  };

  //Add conditional row styles
  const getRowStyle = (status: string) => {
    switch (status.toLowerCase()) {
      case "resolved":
        // return { backgroundColor: "#d9d9d9", color: "#a6a6a6"}; // Grey for closed
        return {color: "#a6a6a6"}; // Grey for closed
      default:
        return {};
    }
  };

  const rows = filteredTickets.map((ticket) => (
    <Table.Tr key={ticket.ticket_id} style={getRowStyle(ticket.status)}> {/* added for coloured grey */}
    {/* <Table.Tr key={ticket.ticket_id}> */}
      <Table.Td>{ticket.ticket_id}</Table.Td>
      <Table.Td>{ticket.category_of_request}</Table.Td>
      <Table.Td>{ticket.nature_of_request}</Table.Td>
      <Table.Td>{ticket.created_date}</Table.Td>
      <Table.Td>{ticket.status}</Table.Td>
      <Table.Td>
        <Group>
          <ActionIcon style={getRowStyle(ticket.status)}
            variant="subtle" color="blue"
            onClick={() => handleOpenMessage(ticket.message,ticket.status)}
          >
            <Tooltip label="Ticket History">
              <IconMessage />
            </Tooltip>
            {/* design for badge */}
            
          </ActionIcon>
        </Group>
      </Table.Td>

      <Table.Td>
        <Group gap="md" wrap="nowrap" justify="center">
          <ActionIcon  style={getRowStyle(ticket.status)}
          variant="subtle" 
          onClick={() => handleOpenDetails(ticket.ticket_id)}>
            <Tooltip label="Details of the Ticket">
              <IconEye />
            </Tooltip>
          </ActionIcon>
          
          {/* Remove delete icon */}
          {/* <ActionIcon style={getRowStyle(ticket.status)}
          disabled={ticket.status.toLowerCase()==='resolved'}
           variant="subtle" color="red"
           onClick={() => handleDelete(ticket.ticket_id)}>
            <Tooltip label="Delete the Ticket">
              <IconTrash/>
            </Tooltip>
          </ActionIcon> */}
        </Group>
      </Table.Td>
    </Table.Tr>

  ));
  return (
    <Container fluid>
      <Title order={3}>View Ticket</Title>
      <Space h="1rem" />
      <Flex align="center" justify="space-between" w={720}>
        <Flex align="center">
          <Title order={5}>Tickets</Title>
          <Space w="1rem" />
          <TextInput
            placeholder="Search"
            value={searchQuery}
            onChange={(event) => setSearchQuery(event.currentTarget.value)}
            radius="md"
            w={250}
            leftSection={<IconSearch size={16} />}
          />
        </Flex>
      </Flex>
      <Space h="1.5rem" />
      <Table
        w={720}
        stickyHeader
        stickyHeaderOffset={60}
        withTableBorder
        highlightOnHover
        horizontalSpacing="lg"
      >
        <Table.Thead>
          <Table.Tr>
            <Table.Th>Ticket ID</Table.Th>
            <Table.Th>Category</Table.Th>
            <Table.Th>Description</Table.Th>
            <Table.Th>Timestamp</Table.Th>
            <Table.Th>Status</Table.Th>
            <Table.Th>History</Table.Th>
            <Table.Th>Action</Table.Th>
          </Table.Tr>
        </Table.Thead>
        <Table.Tbody>{rows}</Table.Tbody>
      </Table>

      <Modal opened={opened} onClose={close} title="">
        <Flex justify="space-between" align="center" >
          <Title order={4}>Ticket History</Title>
          <ActionIcon
            variant="subtle"
            onClick={() => setSortOrder(sortOrder === "asc" ? "desc" : "asc")}
          >
            <Tooltip label={`Sort by ${sortOrder === "asc" ? "Newest " : "Oldest"}`} position="right">
              <IconArrowsUpDown size={18}  />
            </Tooltip>
          </ActionIcon>
        </Flex>
        <Flex direction="column" gap="sm" style={{ maxHeight: "400px", overflowY: "auto" }}>
          {sortedMessages?.map((msg, index) => (
            <div
              key={index}
              style={{
                padding: "10px",
                background: index % 2 === 0 ? "#f1f3f5" : "#dce6f2",
                borderRadius: "10px",
                maxWidth: "80%",
                alignSelf: "flex-start",
                boxShadow: "0 2px 5px rgba(0, 0, 0, 0.1)",
              }}
            >
              <p style={{ margin: 0, fontSize: "14px", color: "black",textAlign: "left" }}>{msg.note}</p>
              {/* //Fetch the note IST time */}
              <p style={{ margin: "6px 0 0", fontSize: "10px", color: "grey", textAlign: "right" }}>Internal team send by {new Date(Number(msg.time)*1000).toLocaleString('en-IN',{timeZone: 'Asia/Kolkata'})}</p>
            </div>
          ))}
          {ticketStatus?.toLowerCase()=== "resolved" && (<div style={{ textAlign: "center", fontSize: "14px", color: "red", marginTop: "10px" }}>
            <p>----- Ended Chat ------</p>
          </div>)}
          {/* Fixed Close Button */}
          <div
            style={{
              position: "sticky",
              bottom: 0,
              background: "white",
              padding: "10px",
              textAlign: "center",
              boxShadow: "0px -5px 4px rgb(255, 255, 255)",
            }}
          >
          <Button onClick={close}>Close</Button>
          </div>
        </Flex>
      </Modal>
 
      <Modal opened={detailsOpened} onClose={closeDetails}>
        {ticketDetails ? (  
          <Flex direction="column" gap="md">
          <div><strong>Ticket ID:</strong> {ticketDetails.ticket_id}</div>
          <div><strong>Category of Complaint:</strong> {ticketDetails.category_of_request}</div>
          <div><strong>Description:</strong> {ticketDetails.nature_of_request}</div>
          {/* <div><strong>Additional Information:</strong> {ticketDetails.additional_info}</div> */}
          {/* Additional Info Table */}
          <div><strong>Additional Info:</strong></div>
          <Table withTableBorder withColumnBorders highlightOnHover>
            <Table.Tbody>
              {parseAdditionalInfo(ticketDetails.additional_info).map((item, index) => (
                <Table.Tr key={index}>
                  <Table.Td>{item.key}</Table.Td>
                  <Table.Td>{item.value}</Table.Td>
                </Table.Tr>
              ))}
            </Table.Tbody>
          </Table>
          <div><strong>Message:</strong> {ticketDetails.message}</div>
          <div><strong>Created Date:</strong> {ticketDetails.created_date}</div>
          <Button onClick={closeDetails} mt="sm">Close</Button>
          </Flex>
        ) : (
          <p>Loading details...</p>
        )}
      </Modal>
    </Container>
  );
}
